@@ -1,5 +1,8 @@
-import composite
-from exception import ProphyError
+from .base_array import base_array
+from .composite import struct, union
+from .exception import ProphyError
+from .six import xrange
+
 
 def decode_scalar_array(tp, data, pos, endianness, count):
     if count is None:
@@ -13,6 +16,7 @@ def decode_scalar_array(tp, data, pos, endianness, count):
         values.append(value)
     return values, cursor
 
+
 def scalar_array_eq(self, other):
     if self is other:
         return True
@@ -22,6 +26,7 @@ def scalar_array_eq(self, other):
     # We are presumably comparing against some other sequence type.
     return other == self._values
 
+
 def composite_array_eq(self, other):
     if self is other:
         return True
@@ -30,32 +35,6 @@ def composite_array_eq(self, other):
                           'other repeated composite fields.')
     return self._values == other._values
 
-class base_array(object):
-    __slots__ = ['_values']
-
-    def __init__(self):
-        self._values = []
-
-    def __getitem__(self, idx):
-        return self._values[idx]
-
-    def __getslice__(self, start, stop):
-        return self._values[start:stop]
-
-    def __len__(self):
-        return len(self._values)
-
-    def __ne__(self, other):
-        return not self == other
-
-    def __hash__(self):
-        raise TypeError('unhashable object')
-
-    def __repr__(self):
-        return repr(self._values)
-
-    def sort(self, sort_function = cmp):
-        self._values.sort(sort_function)
 
 class fixed_scalar_array(base_array):
     __slots__ = []
@@ -65,8 +44,11 @@ class fixed_scalar_array(base_array):
         self._values = [self._TYPE._DEFAULT] * self._max_len
 
     def __setitem__(self, idx, value):
-        value = self._TYPE._check(value)
-        self._values[idx] = value
+        if isinstance(idx, slice):
+            self.__setslice__(idx.start, idx.stop, value)
+        else:
+            value = self._TYPE._check(value)
+            self._values[idx] = value
 
     def __setslice__(self, start, stop, values):
         if len(self._values[start:stop]) != len(values):
@@ -77,12 +59,13 @@ class fixed_scalar_array(base_array):
         return scalar_array_eq(self, other)
 
     def _encode_impl(self, endianness):
-        return "".join(self._TYPE._encode(value, endianness) for value in self)
+        return b"".join(self._TYPE._encode(value, endianness) for value in self)
 
     def _decode_impl(self, data, pos, endianness, _):
         self[:], size = decode_scalar_array(self._TYPE, data, pos, endianness, len(self))
         return size
 
+
 class bound_scalar_array(base_array):
     __slots__ = []
 
@@ -112,8 +95,11 @@ class bound_scalar_array(base_array):
         self._values.remove(elem)
 
     def __setitem__(self, idx, value):
-        value = self._TYPE._check(value)
-        self._values[idx] = value
+        if isinstance(idx, slice):
+            self.__setslice__(idx.start, idx.stop, value)
+        else:
+            value = self._TYPE._check(value)
+            self._values[idx] = value
 
     def __setslice__(self, start, stop, values):
         if self._max_len and len(self) + len(values) - len(self._values[start:stop]) > self._max_len:
@@ -130,7 +116,7 @@ class bound_scalar_array(base_array):
         return scalar_array_eq(self, other)
 
     def _encode_impl(self, endianness):
-        return "".join(self._TYPE._encode(value, endianness) for value in self).ljust(self._SIZE, "\x00")
+        return b"".join(self._TYPE._encode(value, endianness) for value in self).ljust(self._SIZE, b"\x00")
 
     def _decode_impl(self, data, pos, endianness, len_hint):
         if self._SIZE > (len(data) - pos):
@@ -138,8 +124,8 @@ class bound_scalar_array(base_array):
         self[:], size = decode_scalar_array(self._TYPE, data, pos, endianness, len_hint)
         return max(size, self._SIZE)
 
-class fixed_composite_array(base_array):
 
+class fixed_composite_array(base_array):
     __slots__ = []
 
     def __init__(self):
@@ -150,27 +136,33 @@ class fixed_composite_array(base_array):
         return composite_array_eq(self, other)
 
     def _encode_impl(self, endianness):
-        return "".join(value.encode(endianness, terminal = False) for value in self)
+        return b"".join(value.encode(endianness) for value in self)
 
     def _decode_impl(self, data, pos, endianness, _):
         cursor = 0
         for elem in self:
-            cursor += elem._decode_impl(data, pos + cursor, endianness, terminal = False)
+            cursor += elem._decode_impl(data, pos + cursor, endianness, terminal=False)
         return cursor
 
+
 class bound_composite_array(base_array):
     __slots__ = []
 
     def __init__(self):
         super(bound_composite_array, self).__init__()
 
-    """ TODO kl. implement **kwargs to fill structure with data at addition time already """
-    def add(self):
+    def add(self, **attributes):
         if self._max_len and len(self) == self._max_len:
             raise ProphyError("exceeded array limit")
 
         new_element = self._TYPE()
         self._values.append(new_element)
+        for name, value in attributes.items():
+            attr = getattr(new_element, name)
+            if isinstance(attr, base_array):
+                attr[:] = value
+            else:
+                setattr(new_element, name, value)
         return new_element
 
     def extend(self, elem_seq):
@@ -193,7 +185,7 @@ class bound_composite_array(base_array):
         return composite_array_eq(self, other)
 
     def _encode_impl(self, endianness):
-        return "".join(value.encode(endianness, terminal = False) for value in self).ljust(self._SIZE, "\x00")
+        return b"".join(value.encode(endianness) for value in self).ljust(self._SIZE, b"\x00")
 
     def _decode_impl(self, data, pos, endianness, len_hint):
         if self._SIZE > (len(data) - pos):
@@ -202,34 +194,35 @@ class bound_composite_array(base_array):
         cursor = 0
         if not self._SIZE and not self._BOUND:
             while (pos + cursor) < len(data):
-                cursor += self.add()._decode_impl(data, pos + cursor, endianness, terminal = False)
+                cursor += self.add()._decode_impl(data, pos + cursor, endianness, terminal=False)
         else:
             for _ in xrange(len_hint):
-                cursor += self.add()._decode_impl(data, pos + cursor, endianness, terminal = False)
+                cursor += self.add()._decode_impl(data, pos + cursor, endianness, terminal=False)
         return max(cursor, self._SIZE)
 
-def array(type, **kwargs):
+
+def array(type_, **kwargs):
     size = kwargs.pop("size", 0)
     bound = kwargs.pop("bound", None)
     shift = kwargs.pop("shift", 0)
     if kwargs:
         raise ProphyError("unknown arguments to array field")
 
-    if issubclass(type, base_array):
+    if issubclass(type_, base_array):
         raise ProphyError("array of arrays not allowed")
-    if issubclass(type, str):
+    if issubclass(type_, bytes):
         raise ProphyError("array of strings not allowed")
-    if size and type._DYNAMIC:
+    if size and type_._DYNAMIC:
         raise ProphyError("static/limited array of dynamic type not allowed")
     if shift and (not bound or size):
         raise ProphyError("only shifting bound array implemented")
-    if type._UNLIMITED:
+    if type_._UNLIMITED:
         raise ProphyError("array with unlimited field disallowed")
-    if type._OPTIONAL:
+    if type_._OPTIONAL:
         raise ProphyError("array of optional type not allowed")
 
     is_static = size and not bound
-    is_composite = issubclass(type, (composite.struct, composite.union))
+    is_composite = issubclass(type_, (struct, union))
 
     if is_composite:
         base = fixed_composite_array if is_static else bound_composite_array
@@ -239,12 +232,12 @@ def array(type, **kwargs):
     class _array(base):
         __slots__ = []
         _max_len = size
-        _TYPE = type
-        _SIZE = size * type._SIZE
+        _TYPE = type_
+        _SIZE = size * type_._SIZE
         _DYNAMIC = not size
         _UNLIMITED = not size and not bound
         _OPTIONAL = False
-        _ALIGNMENT = type._ALIGNMENT
+        _ALIGNMENT = type_._ALIGNMENT
         _BOUND = bound
         _BOUND_SHIFT = shift
         _PARTIAL_ALIGNMENT = None
