@@ -1,11 +1,12 @@
-import scalar
-import container
-from exception import ProphyError
+from . import scalar
+from .base_array import base_array
+from .exception import ProphyError
+
 
 def optional(cls):
-    if issubclass(cls, str):
+    if issubclass(cls, bytes):
         raise ProphyError("optional bytes not implemented")
-    if issubclass(cls, container.base_array):
+    if issubclass(cls, base_array):
         raise ProphyError("optional array not implemented")
     if cls._DYNAMIC:
         raise ProphyError("optional dynamic fields not implemented")
